/**
 * Title:        CoCompose
 * Copyright:    Copyright (c) 2002, 2003
 * Author:       Dennis Wagelaar
 * Organisation: University of Twente, Vrije Universiteit Brussel
 * Description:  CoCompose Design Language Tool
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License, version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  (See the file "COPYING" that is included with this source distribution.)
 */

package EDU.utwente.CoCompose.gui;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import java.io.*;
import java.util.*;
import EDU.utwente.CoCompose.translate.*;
import EDU.utwente.CoCompose.util.*;

/**
 * A dialog window for editing the implementation generators list of Composites.
 *
 * @author Dennis Wagelaar
 * @version 0.1.10, 22 July 2003
 */

public class ImplGenDialog extends CoComposeDialog
{
  //////////////////////////////////////////////////////////////////////////////
  // Attributes
  //////////////////////////////////////////////////////////////////////////////

  private EDU.utwente.CoCompose.lang.Composite composite_;
  private Vector implementations_ = new Vector();
  JPanel panel1 = new JPanel();
  BorderLayout borderLayout1 = new BorderLayout();
  JPanel implementationsPanel = new JPanel();
  JPanel buttonPanel = new JPanel();
  JButton okBtn = new JButton();
  FlowLayout flowLayout1 = new FlowLayout();
  JLabel jLabel1 = new JLabel();
  JPanel languagePanel = new JPanel();
  BorderLayout borderLayout2 = new BorderLayout();
  JPanel implementationsPanel2 = new JPanel();
  JPanel filenamePanel = new JPanel();
  BorderLayout borderLayout3 = new BorderLayout();
  JLabel jLabel2 = new JLabel();
  JButton browseBtn = new JButton();
  JPanel implementationsPanel3 = new JPanel();
  BorderLayout borderLayout4 = new BorderLayout();
  JPanel addRemovePanel = new JPanel();
  JButton addBtn = new JButton();
  JButton removeBtn = new JButton();
  JTextField languageEdit = new JTextField();
  JTextField filenameEdit = new JTextField();
  FlowLayout flowLayout2 = new FlowLayout();
  FlowLayout flowLayout3 = new FlowLayout();
  FlowLayout flowLayout4 = new FlowLayout();
  JPanel listPanel = new JPanel();
  FlowLayout flowLayout5 = new FlowLayout();
  JScrollPane jScrollPane1 = new JScrollPane();
  JLabel jLabel3 = new JLabel();
  JList implementationsList = new JList();
  JButton upBtn = new JButton();
  JButton downBtn = new JButton();

  //////////////////////////////////////////////////////////////////////////////
  // Constructors
  //////////////////////////////////////////////////////////////////////////////

  /**
   * Constructs a ConceptPropertiesDialog for a Concept object.
   *
   * @param frame The parent frame.
   * @param title The dialog title.
   * @param modal Shows the dialog in modal mode when true.
   * @param composite The Composite of which the implementation generators will be edited.
   */
  public ImplGenDialog(Frame frame, String title, boolean modal,
    EDU.utwente.CoCompose.lang.Composite composite)
  {
    super(frame, title, modal);
    composite_ = composite;
    try
    {
      jbInit();
      pack();
    }
    catch(Exception ex)
    {
      ex.printStackTrace();
    }
  } //ImplGenDialog(Frame frame, String title, boolean modal, Concept concept)

  /**
   * Constructs a ConceptPropertiesDialog.
   * This constructor is only for the JBuilder Designer, do not use it!
   */
  ImplGenDialog()
  {
    this(null, "", false, null);
  } //ImplGenDialog()

  //////////////////////////////////////////////////////////////////////////////
  // Private methods
  //////////////////////////////////////////////////////////////////////////////

  /* JBuilder Component initialization. */
  void jbInit() throws Exception
  {
    panel1.setLayout(borderLayout1);
    okBtn.setText("OK");
    okBtn.addActionListener(new java.awt.event.ActionListener()
    {

      public void actionPerformed(ActionEvent e)
      {
        okBtn_actionPerformed(e);
      }
    });
    buttonPanel.setLayout(flowLayout1);
    jLabel1.setMaximumSize(new Dimension(200, 17));
    jLabel1.setMinimumSize(new Dimension(200, 17));
    jLabel1.setPreferredSize(new Dimension(200, 17));
    jLabel1.setText("Language:");
    implementationsPanel.setLayout(borderLayout2);
    implementationsPanel2.setLayout(borderLayout3);
    jLabel2.setText("File name:");
    jLabel2.setPreferredSize(new Dimension(200, 17));
    jLabel2.setMinimumSize(new Dimension(200, 17));
    jLabel2.setMaximumSize(new Dimension(200, 17));
    browseBtn.setText("Browse");
    browseBtn.addActionListener(new java.awt.event.ActionListener()
    {

      public void actionPerformed(ActionEvent e)
      {
        browseBtn_actionPerformed(e);
      }
    });
    implementationsPanel3.setLayout(borderLayout4);
    addBtn.setText("Add");
    addBtn.addActionListener(new java.awt.event.ActionListener()
    {

      public void actionPerformed(ActionEvent e)
      {
        addBtn_actionPerformed(e);
      }
    });
    removeBtn.setText("Remove");
    removeBtn.addActionListener(new java.awt.event.ActionListener()
    {

      public void actionPerformed(ActionEvent e)
      {
        removeBtn_actionPerformed(e);
      }
    });
    languageEdit.setFont(new java.awt.Font("Dialog", 0, 12));
    languageEdit.setPreferredSize(new Dimension(400, 21));
    filenameEdit.setBackground(getBackground());
    filenameEdit.setFont(new java.awt.Font("Dialog", 0, 12));
    filenameEdit.setPreferredSize(new Dimension(318, 21));
    filenameEdit.setEditable(false);
    filenamePanel.setLayout(flowLayout2);
    languagePanel.setLayout(flowLayout3);
    addRemovePanel.setLayout(flowLayout4);
    listPanel.setLayout(flowLayout5);
    jScrollPane1.setBorder(languageEdit.getBorder());
    jScrollPane1.setPreferredSize(new Dimension(400, 100));
    jLabel3.setMaximumSize(new Dimension(200, 17));
    jLabel3.setMinimumSize(new Dimension(200, 17));
    jLabel3.setPreferredSize(new Dimension(200, 17));
    jLabel3.setText("Installed implementation generators:");
    implementationsList.addListSelectionListener(new javax.swing.event.ListSelectionListener()
    {

      public void valueChanged(ListSelectionEvent e)
      {
        implementationsList_valueChanged(e);
      }
    });
    upBtn.setText("Move up");
    upBtn.addActionListener(new java.awt.event.ActionListener()
    {

      public void actionPerformed(ActionEvent e)
      {
        upBtn_actionPerformed(e);
      }
    });
    downBtn.setText("Move down");
    downBtn.addActionListener(new java.awt.event.ActionListener()
    {

      public void actionPerformed(ActionEvent e)
      {
        downBtn_actionPerformed(e);
      }
    });
    panel1.setBorder(BorderFactory.createEtchedBorder());
    getContentPane().add(panel1);
    panel1.add(implementationsPanel, BorderLayout.CENTER);
    implementationsPanel.add(languagePanel, BorderLayout.NORTH);
    languagePanel.add(jLabel1, null);
    languagePanel.add(languageEdit, null);
    implementationsPanel.add(implementationsPanel2, BorderLayout.CENTER);
    implementationsPanel2.add(filenamePanel, BorderLayout.NORTH);
    filenamePanel.add(jLabel2, null);
    filenamePanel.add(filenameEdit, null);
    filenamePanel.add(browseBtn, null);
    implementationsPanel2.add(implementationsPanel3, BorderLayout.CENTER);
    implementationsPanel3.add(addRemovePanel, BorderLayout.NORTH);
    addRemovePanel.add(addBtn, null);
    addRemovePanel.add(removeBtn, null);
    addRemovePanel.add(upBtn, null);
    addRemovePanel.add(downBtn, null);
    implementationsPanel3.add(listPanel, BorderLayout.CENTER);
    listPanel.add(jLabel3, null);
    listPanel.add(jScrollPane1, null);
    jScrollPane1.getViewport().add(implementationsList, null);
    panel1.add(buttonPanel, BorderLayout.SOUTH);
    buttonPanel.add(okBtn, null);
  } //jbInit

  /* Reads implementation generators from composite into listbox. */
  private void readImplementations()
  {
    ImplementationGenerator implementation;
    Vector implementationNames = new Vector();

    implementationsList.removeAll();
    implementations_.clear();
    if (composite_ == null)
      return;

    Enumeration implementations = composite_.getImplementations();
    while (implementations.hasMoreElements())
    {
      implementation = (ImplementationGenerator) implementations.nextElement();
      implementations_.add(implementation);
      implementationNames.add(implementation.getLanguage() + ", " +
        implementation.getFilename());
    }
    implementationsList.setListData(implementationNames);
  } //readimplementations

  /* Browse for a file name. */
  void browseBtn_actionPerformed(ActionEvent e)
  {
    // browse for any file
    JFileChooser browseDlg = new JFileChooser("./implementations");
    DirFileFilter filter = new DirFileFilter(".xml");
    browseDlg.setDialogTitle("Browse for implementation generator file");
    browseDlg.setFileFilter(filter);
    if (browseDlg.showOpenDialog(this) != JFileChooser.APPROVE_OPTION)
      return;
    File file = browseDlg.getSelectedFile();
    // set file name edit box
    try
    {
      String path = file.getAbsolutePath();
      File pathF = new File("./");
      path = path.substring(pathF.getAbsolutePath().length() - 2);
      filenameEdit.setText("." + path.replace('\\', '/'));
    }
    catch (Exception exc)
    {
      reportError("The file must be stored in the CoCompose directory or one " +
       "of its subdirectories", "Error", null);
    }
  } //browseBtn_actionPerformed

  /* Adds a implementation generator to the composite. */
  void addBtn_actionPerformed(ActionEvent e)
  {
    // create new implementation generator
    ImplementationGenerator implementation = new ImplementationGenerator();
    implementation.setLanguage(languageEdit.getText());
    try
    {
      implementation.setFilename(filenameEdit.getText());
    }
    catch (CoComposeException exc)
    {
      reportError("Implementation generator file could not be parsed: " +
        exc.getMessage(), "Error", exc.getConcept());
      return;
    }
    // check if implementation ok
    if (implementation.getLanguage().length() == 0)
    {
      reportError("Implementation generator language must be set", "Error", null);
      return;
    }
    if (implementation.getFilename().length() == 0)
    {
      reportError("Implementation generator filename must be set", "Error", null);
      return;
    }
    composite_.addImplementation(implementation);
    readImplementations();
    implementationsList.setSelectedIndex(implementations_.size() - 1);
  } //addBtn_actionPerformed

  /* Removes a implementation generator from the composite. */
  void removeBtn_actionPerformed(ActionEvent e)
  {
    ImplementationGenerator implementation;

    if (implementationsList.getSelectedIndex() < 0)
      return;

    implementation = (ImplementationGenerator) implementations_.get(implementationsList.getSelectedIndex());
    composite_.removeImplementation(implementation);
    readImplementations();
  } //removeBtn_actionPerformed

  /* Closes window. */
  void okBtn_actionPerformed(ActionEvent e)
  {
    dispose();
  } //okBtn_actionPerformed

  /* Other list item selected. */
  void implementationsList_valueChanged(ListSelectionEvent e)
  {
    ImplementationGenerator implementation;

    if (implementationsList.getSelectedIndex() < 0)
      return;

    implementation = (ImplementationGenerator) implementations_.get(implementationsList.getSelectedIndex());
    languageEdit.setText(implementation.getLanguage());
    filenameEdit.setText(implementation.getFilename());
  } //implementationsList_valueChanged

  /* Move up selected implementation patterm. */
  void upBtn_actionPerformed(ActionEvent e)
  {
    ImplementationGenerator implementation;
    int index = implementationsList.getSelectedIndex();

    if (index < 1)
      return;

    implementation = (ImplementationGenerator) implementations_.get(index);
    composite_.removeImplementation(implementation);
    composite_.insertImplementation(implementation, index - 1);
    readImplementations();
    implementationsList.setSelectedIndex(index - 1);
  } //upBtn_actionPerformed

  /* Move down selected implementation patterm. */
  void downBtn_actionPerformed(ActionEvent e)
  {
    ImplementationGenerator implementation;
    int index = implementationsList.getSelectedIndex();

    if ((index < 0) || (index >= implementations_.size() - 1))
      return;

    implementation = (ImplementationGenerator) implementations_.get(index);
    composite_.removeImplementation(implementation);
    composite_.insertImplementation(implementation, index + 1);
    readImplementations();
    implementationsList.setSelectedIndex(index + 1);
  } //downBtn_actionPerformed

  //////////////////////////////////////////////////////////////////////////////
  // Public methods
  //////////////////////////////////////////////////////////////////////////////

  /**
   * Shows dialog window.
   */
  public void show()
  {
    // read implementation generators into list
    readImplementations();
    super.show();
  } //show
} //class ImplGenDialog
