/**
 * Title:        CoCompose
 * Copyright:    Copyright (c) 2002, 2003
 * Author:       Dennis Wagelaar
 * Organisation: University of Twente, Vrije Universiteit Brussel
 * Description:  CoCompose Design Language Tool
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License, version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  (See the file "COPYING" that is included with this source distribution.)
 */

package EDU.utwente.CoCompose.gui;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import java.io.*;
import java.util.*;
import EDU.utwente.CoCompose.translate.*;
import EDU.utwente.CoCompose.util.*;

/**
 * A dialog window for editing the solution patterns list of Composites.
 *
 * @author Dennis Wagelaar
 * @version 0.1.10, 22 July 2003
 */

public class SolPattDialog extends CoComposeDialog
{
  //////////////////////////////////////////////////////////////////////////////
  // Attributes
  //////////////////////////////////////////////////////////////////////////////

  private EDU.utwente.CoCompose.lang.Composite composite_;
  private Vector solutions_ = new Vector();
  JPanel panel1 = new JPanel();
  BorderLayout borderLayout1 = new BorderLayout();
  JPanel solutionsPanel = new JPanel();
  JPanel buttonPanel = new JPanel();
  JButton okBtn = new JButton();
  FlowLayout flowLayout1 = new FlowLayout();
  BorderLayout borderLayout2 = new BorderLayout();
  JButton browseBtn = new JButton();
  FlowLayout flowLayout2 = new FlowLayout();
  JPanel filenamePanel = new JPanel();
  JTextField filenameEdit = new JTextField();
  JLabel jLabel2 = new JLabel();
  JButton addBtn = new JButton();
  JScrollPane jScrollPane1 = new JScrollPane();
  JButton downBtn = new JButton();
  JPanel solutionsPanel2 = new JPanel();
  FlowLayout flowLayout5 = new FlowLayout();
  BorderLayout borderLayout3 = new BorderLayout();
  FlowLayout flowLayout4 = new FlowLayout();
  JPanel listPanel = new JPanel();
  JButton upBtn = new JButton();
  JLabel jLabel3 = new JLabel();
  JPanel addRemovePanel = new JPanel();
  JButton removeBtn = new JButton();
  JList solutionsList = new JList();

  //////////////////////////////////////////////////////////////////////////////
  // Constructors
  //////////////////////////////////////////////////////////////////////////////

  /**
   * Constructs a SolPattDialog for a Composite object.
   *
   * @param frame The parent frame.
   * @param title The dialog title.
   * @param modal Shows the dialog in modal mode when true.
   * @param composite The Composite of which the solution patterns will be edited.
   */
  public SolPattDialog(Frame frame, String title, boolean modal,
    EDU.utwente.CoCompose.lang.Composite composite)
  {
    super(frame, title, modal);
    composite_ = composite;
    try
    {
      jbInit();
      pack();
    }
    catch(Exception ex)
    {
      ex.printStackTrace();
    }
  } //SolPattDialog(Frame frame, String title, boolean modal, Concept concept)

  /**
   * Constructs a SolPattDialog.
   * This constructor is only for the JBuilder Designer, do not use it!
   */
  SolPattDialog()
  {
    this(null, "", false, null);
  } //SolPattDialog()

  //////////////////////////////////////////////////////////////////////////////
  // Private methods
  //////////////////////////////////////////////////////////////////////////////

  /* JBuilder Component initialization. */
  void jbInit() throws Exception
  {
    panel1.setLayout(borderLayout1);
    okBtn.setText("OK");
    okBtn.addActionListener(new java.awt.event.ActionListener()
    {

      public void actionPerformed(ActionEvent e)
      {
        okBtn_actionPerformed(e);
      }
    });
    buttonPanel.setLayout(flowLayout1);
    solutionsPanel.setLayout(borderLayout2);
    panel1.setBorder(BorderFactory.createEtchedBorder());
    browseBtn.setText("Browse");
    browseBtn.addActionListener(new java.awt.event.ActionListener()
    {

      public void actionPerformed(ActionEvent e)
      {
        browseBtn_actionPerformed(e);
      }
    });
    filenamePanel.setLayout(flowLayout2);
    filenameEdit.setBackground(getBackground());
    filenameEdit.setFont(new java.awt.Font("Dialog", 0, 12));
    filenameEdit.setPreferredSize(new Dimension(318, 21));
    filenameEdit.setEditable(false);
    jLabel2.setText("File name:");
    jLabel2.setPreferredSize(new Dimension(200, 17));
    jLabel2.setMinimumSize(new Dimension(200, 17));
    jLabel2.setMaximumSize(new Dimension(200, 17));
    addBtn.setText("Add");
    addBtn.addActionListener(new java.awt.event.ActionListener()
    {

      public void actionPerformed(ActionEvent e)
      {
        addBtn_actionPerformed(e);
      }
    });
    jScrollPane1.setBorder(filenameEdit.getBorder());
    jScrollPane1.setPreferredSize(new Dimension(400, 100));
    downBtn.setText("Move down");
    downBtn.addActionListener(new java.awt.event.ActionListener()
    {

      public void actionPerformed(ActionEvent e)
      {
        downBtn_actionPerformed(e);
      }
    });
    solutionsPanel2.setLayout(borderLayout3);
    listPanel.setLayout(flowLayout5);
    upBtn.setText("Move up");
    upBtn.addActionListener(new java.awt.event.ActionListener()
    {

      public void actionPerformed(ActionEvent e)
      {
        upBtn_actionPerformed(e);
      }
    });
    jLabel3.setMaximumSize(new Dimension(200, 17));
    jLabel3.setMinimumSize(new Dimension(200, 17));
    jLabel3.setPreferredSize(new Dimension(200, 17));
    jLabel3.setText("Installed solution patterns:");
    addRemovePanel.setLayout(flowLayout4);
    removeBtn.setText("Remove");
    removeBtn.addActionListener(new java.awt.event.ActionListener()
    {

      public void actionPerformed(ActionEvent e)
      {
        removeBtn_actionPerformed(e);
      }
    });
    solutionsList.addListSelectionListener(new javax.swing.event.ListSelectionListener()
    {

      public void valueChanged(ListSelectionEvent e)
      {
        solutionsList_valueChanged(e);
      }
    });
    getContentPane().add(panel1);
    panel1.add(solutionsPanel, BorderLayout.CENTER);
    solutionsPanel.add(filenamePanel,  BorderLayout.NORTH);
    filenamePanel.add(jLabel2, null);
    filenamePanel.add(filenameEdit, null);
    filenamePanel.add(browseBtn, null);
    solutionsPanel.add(solutionsPanel2, BorderLayout.CENTER);
    addRemovePanel.add(addBtn, null);
    addRemovePanel.add(removeBtn, null);
    addRemovePanel.add(upBtn, null);
    addRemovePanel.add(downBtn, null);
    solutionsPanel2.add(listPanel, BorderLayout.CENTER);
    solutionsPanel2.add(addRemovePanel, BorderLayout.NORTH);
    listPanel.add(jLabel3, null);
    listPanel.add(jScrollPane1, null);
    jScrollPane1.getViewport().add(solutionsList, null);
    panel1.add(buttonPanel, BorderLayout.SOUTH);
    buttonPanel.add(okBtn, null);
  } //jbInit

  /* Reads solution patterns from composite into listbox. */
  private void readSolutions()
  {
    SolutionPattern solution;
    Vector solutionNames = new Vector();

    solutionsList.removeAll();
    solutions_.clear();
    if (composite_ == null)
      return;

    Enumeration solutions = composite_.getSolutions();
    while (solutions.hasMoreElements())
    {
      solution = (SolutionPattern) solutions.nextElement();
      solutions_.add(solution);
      solutionNames.add(solution.getFilename());
    }
    solutionsList.setListData(solutionNames);
  } //readSolutions

  /* Browse for a file name. */
  void browseBtn_actionPerformed(ActionEvent e)
  {
    // browse for any file
    JFileChooser browseDlg = new JFileChooser("./models");
    DirFileFilter filter = new DirFileFilter(".xml");
    browseDlg.setDialogTitle("Browse for solution pattern file");
    browseDlg.setFileFilter(filter);
    if (browseDlg.showOpenDialog(this) != JFileChooser.APPROVE_OPTION)
      return;
    File file = browseDlg.getSelectedFile();
    // set file name edit box
    try
    {
      String path = file.getAbsolutePath();
      File pathF = new File("./");
      path = path.substring(pathF.getAbsolutePath().length() - 2);
      filenameEdit.setText("." + path.replace('\\', '/'));
    }
    catch (Exception exc)
    {
      reportError("The file must be stored in the CoCompose directory or one " +
       "of its subdirectories", "Error", null);
    }
  } //browseBtn_actionPerformed

  /* Adds a solution pattern to the composite. */
  void addBtn_actionPerformed(ActionEvent e)
  {
    // create new solution pattern
    SolutionPattern solution = new SolutionPattern();
    solution.setFilename(filenameEdit.getText());
    // check if solution ok
    if (solution.getFilename().length() == 0)
    {
      reportError("Solution pattern filename must be set", "Error", null);
      return;
    }
    composite_.addSolution(solution);
    readSolutions();
    solutionsList.setSelectedIndex(solutions_.size() - 1);
  } //addBtn_actionPerformed

  /* Removes a solution pattern from the composite. */
  void removeBtn_actionPerformed(ActionEvent e)
  {
    SolutionPattern solution;

    if (solutionsList.getSelectedIndex() < 0)
      return;

    solution = (SolutionPattern) solutions_.get(solutionsList.getSelectedIndex());
    composite_.removeSolution(solution);
    readSolutions();
  } //removeBtn_actionPerformed

  /* Closes window. */
  void okBtn_actionPerformed(ActionEvent e)
  {
    dispose();
  } //okBtn_actionPerformed

  /* Other list item selected. */
  void solutionsList_valueChanged(ListSelectionEvent e)
  {
    SolutionPattern solution;

    if (solutionsList.getSelectedIndex() < 0)
      return;

    solution = (SolutionPattern) solutions_.get(solutionsList.getSelectedIndex());
    filenameEdit.setText(solution.getFilename());
  } //solutionsList_valueChanged

  /* Move up selected solution patterm. */
  void upBtn_actionPerformed(ActionEvent e)
  {
    SolutionPattern solution;
    int index = solutionsList.getSelectedIndex();

    if (index < 1)
      return;

    solution = (SolutionPattern) solutions_.get(index);
    composite_.removeSolution(solution);
    composite_.insertSolution(solution, index - 1);
    readSolutions();
    solutionsList.setSelectedIndex(index - 1);
  } //upBtn_actionPerformed

  /* Move down selected solution patterm. */
  void downBtn_actionPerformed(ActionEvent e)
  {
    SolutionPattern solution;
    int index = solutionsList.getSelectedIndex();

    if ((index < 0) || (index >= solutions_.size() - 1))
      return;

    solution = (SolutionPattern) solutions_.get(index);
    composite_.removeSolution(solution);
    composite_.insertSolution(solution, index + 1);
    readSolutions();
    solutionsList.setSelectedIndex(index + 1);
  } //downBtn_actionPerformed

  //////////////////////////////////////////////////////////////////////////////
  // Public methods
  //////////////////////////////////////////////////////////////////////////////

  /**
   * Shows dialog window.
   */
  public void show()
  {
    // read solution patterns into list
    readSolutions();
    super.show();
  } //show
} //class SolPattDialog
